﻿var Promise = require('promise');
var jsMapper = require("./js_diff/index.js");
var consoleFileLogger = require("./sl-logger.js").consoleFileLogger;
var version = require("../package.json").version;

function BuildDiffProcess(cfg, diffService, sourceConrolProviders, parsedDependencies, logger) {

    var log = logger? logger.child({className: "BuildDiffProcess"}) : consoleFileLogger("BuildDiffProcess");

    function createBuildDiffPromiseChain(buildArguments) {
        var handlerParams = {
            buildArguments: buildArguments,
            cfg: cfg,
            finalOutput: {
                "meta": {
                    "generated": new Date().valueOf(),
                    "build": buildArguments.build,
                    "commit": buildArguments.commit,
                    "customerId": cfg.customerId,
                    "appName": buildArguments.appname,
                    //"technologies": "nodejs",
                    "technology": "nodejs",
                    "branch": buildArguments.branch,
                    "environment": {agentVersion: version},
                    "authors": buildArguments.author,
                    "logsUrl": buildArguments.logsUrl,
                    "jobName": buildArguments.jobName
                },
                files: [],
                dependencies: parsedDependencies || []
            }
        };

        var scmInstance;

        if (buildArguments.workspacepath) {
            var scmType = sourceConrolProviders[buildArguments.scm];
            if (scmType) {
                scmInstance = new scmType(cfg, buildArguments.workspacePath);
                handlerParams.finalOutput.meta.scm = buildArguments.scm;
            }
        }

        if (!buildArguments.commit && scmInstance && !scmInstance.detectCommitVersion) {
            return Promise.reject("'commit' command-line argument is required, since automatic detection by the " + buildArguments.scm + " SCM is not supported");
        }

        var promiseChain = Promise.resolve();

        if (buildArguments.workspacepath) {

            //2. If there is a mapping, get the diffs
            if (scmInstance) {
                //2.1 Get repository url from the SCM
                promiseChain = promiseChain.then(function (mapping) {
                    return getRepositoryUrl(handlerParams, scmInstance);
                });

                //2.2 Get branch history from the SCM
                promiseChain = promiseChain.then(function (mapping) {
                    return loadBranchHistory(handlerParams, scmInstance);
                });

                //2.3 if commit was not specified, automatically detect it
                if (!buildArguments.commit) {
                    promiseChain = promiseChain.then(function () {
                        return scmInstance.detectCommitVersion(handlerParams, log).then(function (commit) {
                            if (commit) {
                                handlerParams.finalOutput.meta.commit = commit;
                            }
                        });
                    });
                };
            }

            //3. Call JS mapping handler
            promiseChain = promiseChain.then(function () {
                return jsMapper(handlerParams, log);
            });
        };

        //4. Submit the final output to the server
        promiseChain = promiseChain.then(function () {
            return diffService.submitMapping(handlerParams.finalOutput);
        });

        promiseChain = promiseChain.then(function () {
            log.info("Build agent finished");
            return Promise.resolve();
        }).catch(function (err) {
            log.info("Build agent error: " + err);
            return Promise.reject(err);
        });

        return promiseChain;
    }

    this.run = function run(buildArguments) {
        return createBuildDiffPromiseChain(buildArguments);
    }

    function loadBranchHistory(handlerParams, scmInstance) {
        return scmInstance.getBranchHistory(handlerParams, log).then(function (gitLog) {
            if (gitLog && gitLog.commitHistory && gitLog.commitHistory.length && gitLog.commitLog && gitLog.commitLog.length) {
                log.info('Branch history loaded (' + gitLog.length + ' commits)');
                handlerParams.finalOutput.meta.history = gitLog.commitHistory;
                handlerParams.finalOutput.meta.commitLog = gitLog.commitLog;
            }
            return gitLog;
        });
    }

    function getRepositoryUrl(handlerParams, scmInstance) {
        return scmInstance.getRepositoryUrl(handlerParams, log).then(function (repositoryUrl) {
            if (repositoryUrl) {
                log.info('Repository Url loaded (' + repositoryUrl + ')');
                handlerParams.finalOutput.meta.repositoryUrl = repositoryUrl;
            }
            return repositoryUrl;
        });
    }
}
module.exports.BuildDiffProcess = BuildDiffProcess;
module.exports.DiffService = require("./new-diff-service.js").NewDiffService;
module.exports.sourceControlProviders = require("./source-control-providers.js");