﻿var Promise = require('promise'),
    childProcess = require('child_process'),
    spawn = require('child_process').spawn,
    diffParse = require('parse-diff'),
    logParse = require('./parse-log.js').logParse,
    util = require('util'),
    consoleFileLogger = require("../sl-logger.js").consoleFileLogger;

function Git(cfg) {
    this.getCfg = function getCfg() { return cfg; }
}

Git.prototype.getGitPath = function getGitPath() {
    return (this.getCfg() || {}).gitPath || 'git';
    // /tools/arch/Linux_2.6.32-x86_64/gcc-4.9.1-buildrel/bin/git
}

Git.prototype.detectCommitVersion = function detectCommitVersion(handlerParams, logger) {
    var gitPath = this.getGitPath();
    var cfg = this.getCfg();
    var log = logger? logger.child({methodName: "detectCommitVersion"}) : consoleFileLogger("detectCommitVersion");
    var p = new Promise(function (resolve, reject) {
        
        try {
            log.info('GIT: invoking git log -n 1.');
            log.info('GIT PATH: '+gitPath);
            var cmd = gitPath + " log -n 1 --pretty=format:%H";
            var gitLogOutput = childProcess.exec(cmd, {
                cwd: handlerParams.buildArguments.workspacepath,
                //maxBuffer: 1024 * 1024 * 200 //200 Mb, instead of 200Kb
            }, function (error, stdout, stderr) {
                //debuglog('stdout: ' + stdout);
                if (error !== null) {
                    log.error('GIT log error: ' + error);
                    log.error('stderr: ' + stderr);
                    reject(error);
                    return;
                }
                
                var commit = stdout.toString().trim();
                resolve(commit);
            });
            
        } catch (e) {
            log.error('GIT log error: ' + e);
            reject(e);
        }
    });
    return p;
}

Git.prototype.getBranchHistory = function getBranchHistory(handlerParams, logger) {
    var gitPath = this.getGitPath();
    var cfg = this.getCfg();
    var log = logger? logger.child({methodName: "getBranchHistory"}) : consoleFileLogger("getBranchHistory");
    var p = new Promise(function (resolve, reject) {
        var historyLength = 100;
        if (cfg.branchHistoryLength) {
            try {
                var v = parseInt(cfg.branchHistoryLength);
                if (v < 1 || Number.isNaN(v)) {
                    throw new Error(); //will be caught below
                }
                else {
                    historyLength = v;
                }
            }
            catch (e) {
                log.error("An invalid value was specified for branchHistoryLength in sealights.json: " + cfg.branchHistoryLength + ". Valid values are positive integers. Defaulting to " + historyLength);
            }
        }
        
        try {
            
            log.info('GIT: invoking git log to get branch history');
            log.info('GIT PATH: ' + gitPath);
            var format = {
                "commit": "%H",
                "authorName": "%an",
                "authorEmail": "%ae",
                "commiterName": "%cn",
                "commiterEmail": "%ce",
                "authorDate": "%at000",
                "commiterDate": "%ct000",
                "title": "%f"
            };
            var gitLogOutput = childProcess.exec(gitPath + ' log -n ' + historyLength + ' --pretty=format:' + '\'' + JSON.stringify(format) + '\'', {
                cwd: handlerParams.buildArguments.workspacepath,
                maxBuffer: 1024 * 1024 * 200 //200 Mb, instead of 200Kb
            }, function (error, stdout, stderr) {
                if (error !== null) {
                    log.error('GIT log error: ' + error);
                    log.error('stderr: ' + stderr);
                    reject(error);
                    return;
                }
                var commitHistory = [];
                var commitLog = [];
                var lines = stdout.toString().split('\n');
                lines.forEach(function (line) {
                    line = line.trim();
                    try {
                        var gitLogEntry = JSON.parse(line);
                        commitHistory.push(gitLogEntry.commit);
                        commitLog.push(gitLogEntry);
                    }
                    catch (err) {
                        log.error("GIT: Failed parsing git log in json. Error: " + err)
                    }
                });
                var gitLog = {
                    commitHistory: commitHistory,
                    commitLog: commitLog
                };
                resolve(gitLog);
            });
            
        } catch (e) {
            log.error('GIT log error: ' + e);
            reject(e);
        }
    });
    return p;
}

Git.prototype.getRepositoryUrl = function getRepositoryUrl(handlerParams, logger) {
    var gitPath = this.getGitPath();
        var log = logger? logger.child({methodName: "getRepositoryUrl"}) : consoleFileLogger("getRepositoryUrl");

    var p = new Promise(function (resolve, reject) {
        var gitLogOutput = childProcess.exec(gitPath + ' remote -v', {
            cwd: handlerParams.buildArguments.workspacepath,
            maxBuffer: 1024 * 1024 * 200 //200 Mb, instead of 200Kb
        }, function (error, stdout, stderr) {
            if (error !== null) {
                log.error('GIT log error: ' + error);
                log.error('stderr: ' + stderr);
                reject(error);
                return;
            }

            try {
                var repositoryUrl = '';
                if (stdout.toString()) {
                    repositoryUrl = logParse(stdout.toString());
                    repositoryUrl = repositoryUrl[0].split('\t').map(function (l) {return l.trim()})[1].replace(' (fetch)', '');
                }
                resolve(repositoryUrl);
            }
            catch (e) {
                log.error('GIT log error: ' + e);
                resolve('');
            }
        });
    })
    return p;
}

module.exports = Git;