var bunyan = require('bunyan');
var SlNeedle = require("./sl-needle.js").SlNeedle;
var bformat = require('bunyan-format');
var formatStdOut = bformat({levelInString: true}, process.stdout);

var TRACE = 10;
var DEBUG = 20;
var INFO = 30;
var WARN = 40;
var ERROR = 50;
var FATAL = 60;

function mapLevelToName(level) {
  switch (level) {
    case TRACE:
      return 'TRACE';
    case DEBUG:
      return 'DEBUG';
    case INFO:
      return 'INFO';
    case WARN:
      return 'WARN';
    case ERROR:
      return 'ERROR';
    case FATAL:
      return 'FATAL';
  }
}

function BunyanLogger(config){
    this.buffer = [];
    this.bufferLength = config.bufferLength || 50;
    this.bufferTimeout = config.bufferTimeout || 3000;
    this.maxConnections = 10;
    this.currentConnections = 0;
    this.config = config;
    this.needle = new SlNeedle(config.token);
}

BunyanLogger.prototype.write = function(data){
    data.level = mapLevelToName(data.level);
    this.buffer.push(data);
    this.checkBuffer(false, function (err) {});
};

BunyanLogger.prototype.processBuffer = function(callback){
    clearTimeout(this._timeoutId);

    var entries = this.buffer.slice();
    this.buffer = [];
    if (this.currentConnections == this.maxConnections) {
        return callback(null);
    }

    this._sendLogs(entries, function (err) {
        if (err){
            console.error("[LOG SUBMISSION] Failed to send logs to Sealights. Error: " + err);
            callback(err);
        }
        else {
            callback(null);
        }
    });
};

BunyanLogger.prototype.checkBuffer = function(isLastProcess, callback){
    var bunyanLogger = this;

    if (isLastProcess) { // This flag is true only on shutdown
        this.bufferLength = 1;
        this.bufferTimeout = 0;
    }

    if (!this.buffer.length) {
        return callback(null);
    }

    if(this.buffer.length >= this.bufferLength){
        return this.processBuffer(callback);
    }

    if(this.bufferTimeout){
        clearTimeout(this._timeoutId);

        this._timeoutId = setTimeout(
            function(){
                bunyanLogger.processBuffer(callback);
            },
            this.bufferTimeout
        );
    }
};

BunyanLogger.prototype._sendLogs = function sendLogs(entries, callback) {
    var firstEntry = entries[0];
    var data = {
        customerId: firstEntry.customerId,
        appName: firstEntry.appName,
        creationTime: new Date().valueOf(),
        environment: firstEntry.environment,
        log: entries.map(function (entry) {
            return JSON.stringify(entry)
        }).join("\n")
    };
    if (!data.customerId || !data.appName) {
        return callback(new Error("[LOG SUBMISSION] customerId and appName are required to send logs"))
    }

    var opts = {
        json: true,
        compressed: true,
        read_timeout: 60000
    };

    if (this.config.proxy)
        opts.proxy = this.config.proxy;

    this.currentConnections += 1;
    var routeVersion = this.config.isValidToken ? "/v2" : "/v1";
    var url = this.config.server + routeVersion + "/logsubmission";
    var that = this;
    this.needle.post(url, data, opts, function (err, resp, body) {
        that.currentConnections -= 1;
        if (!err && resp && resp.statusCode >= 400) err = new Error("HTTP ERROR: " + resp.statusCode + " " + resp.statusMessage);
        if (err) {
            callback(err);
        }
        else {
            callback(null);
        }
    })
};

var nullLogger = function () {
    var log = bunyan.createLogger({
        name: "CIA",
    });
    return log;
};

var consoleFileLogger = function (name) {
    var loggerConfig = {
        name: name || "CIA",
        streams: []
    };
    if (process.env.NODE_DEBUG == "sl" || process.env.NODE_DEBUG == "sl-console") {
        loggerConfig.streams.push({level: 'debug', stream: formatStdOut})
    }
    if (process.env.NODE_DEBUG == "sl-file") {
        loggerConfig.streams.push({level: 'debug', path: "sl.node.log"})
    }
    return bunyan.createLogger(loggerConfig);
};

var logger = function (config) {
    var remoteStream;
    var environment = {environmentName: "null"};
    if (config.environmentName) {
        environment = {environmentName: config.environmentName}
    }
    if (config.environment && config.environment.environmentName) {
        environment = config.environment;
    }
    var loggerConfig = {
        name: "CIA",
        customerId: config.customerId,
        appName: config.appName,
        streams: []
    };
    if (process.env.NODE_DEBUG == "sl" || process.env.NODE_DEBUG == "sl-console" || config.NODE_DEBUG == "sl") {
        loggerConfig.streams.push({level: 'debug', stream: formatStdOut})
    }
    if (process.env.NODE_DEBUG == "sl-file" || config.NODE_DEBUG == "sl-file") {
        loggerConfig.streams.push({level: 'debug', path: "sl.node.log"})
    }
    if (config.sendLogs) {
        remoteStream = new BunyanLogger(config);
        loggerConfig.streams.push({level: 'debug', type: "raw", stream: remoteStream});
    }
    var bunyanLogger = bunyan.createLogger(loggerConfig);
    bunyanLogger.remoteStream = remoteStream;
    return bunyanLogger
};

module.exports.BunyanLogger = BunyanLogger;
module.exports.logger = logger;
module.exports.nullLogger = nullLogger;
module.exports.consoleFileLogger = consoleFileLogger;